import { ref } from "vue";
import { defineStore } from "pinia";
import dayjs from "dayjs";
import type { ElecCostSummaryMap } from "../utils/types";
import { CalcuEleCost, CalcuEleStandCost } from "../utils/CalcuEleCost";

const useElecReportStore = defineStore("elecReportData", () => {
  const elecData = ref<any[]>([]);
  // @ts-ignore
  let timerId = null;

  // 初始化為空物件，並定義類型
  const elecCostSummary = ref<ElecCostSummaryMap>({
    thisMonth: {},
    lastMonth: {},
  });

  // 儲存每個電表的 dataMap
  const elecDataMaps = ref<{
    [id: string]: {
      thisMonth: Map<string, number>;
      lastMonth: Map<string, number>;
    };
  }>({});

  const startTime = ref("");
  const endTime = ref("");

  // get data from baja
  const getElecDataFromBaja = () => {
    // @ts-ignore
    window.require &&
      // @ts-ignore
      window.requirejs(["baja!"], (baja: any) => {
        console.log("進入 bajaSubscriber 準備執行用電度數 BQL 訂閱");

        // 定義BQL 查詢
        const Total_kwhBql = `local:|foxs:|station:|neql:EMS:SubSys_kwh|bql:select slotPath,parent.displayName,displayName,NumericInterval.historyConfig.id,out`;

        // 執行各電表的 BQL 查詢
        fetchElecData(baja, Total_kwhBql);
      });
  };

  const fetchElecData = (baja: any, bql: string) => {
    let eleclist: any[] = [];
    baja.Ord.make(bql).get({
      cursor: {
        before: () => {},
        each: (record: any) => {
          eleclist.push({
            slotPath: record.get("slotPath"),
            displayName: record.get("parent$2edisplayName"),
            id: record.get("NumericInterval$2ehistoryConfig$2eid").$cEncStr,
            area: record.get("out")?.get("value") ?? "",
          });
        },
        after: () => {
          const validElecList = eleclist.filter(
            (item) => item.id !== undefined
          );
          const menoElecList = eleclist.filter((item) => item.id == undefined);
          const displayNameToAreaMap = new Map<string, string>();
          menoElecList.forEach((item) => {
            displayNameToAreaMap.set(item.displayName, item.area);
          });
          validElecList.forEach((item) => {
            const area = displayNameToAreaMap.get(item.displayName);
            if (area) {
              item.area = area; // 將 area 賦值給 validElecList 中的 item
            }
          });
          elecData.value = [];
          elecData.value.push(...validElecList);
          validElecList.forEach((item) => {
            const id = item.id;
            elecDataMaps.value[id] = {
              thisMonth: new Map<string, number>(),
              lastMonth: new Map<string, number>(),
            };
            // 初始化每个 slotPath 的值为 undefined 或 null
            elecCostSummary.value.thisMonth[id] = undefined;
            elecCostSummary.value.lastMonth[id] = undefined;
            getTimeToHistory(item, 'thisMonth'); // 獲取本月
            getTimeToHistory(item, 'lastMonth'); // 獲取上個月
          });
        },
      },
    });
  };

  const getTimeToHistory = (item: any, month: 'thisMonth' | 'lastMonth') => {
    const id = item.id;
    let startTimeValue = startTime.value;
    let endTimeValue = endTime.value;

    if (month === 'lastMonth') {
      startTimeValue = dayjs(startTime.value).subtract(1, 'month').startOf('month').format("YYYY-MM-DDTHH:mm:ss.000+08:00");
      endTimeValue = dayjs(endTime.value).subtract(1, 'month').endOf('month').format("YYYY-MM-DDTHH:mm:ss.000+08:00");
    }

    const ordString = `local:|foxs:|history:${id}?period=timerange;start=${startTimeValue};end=${endTimeValue}|bql:history:HistoryRollup.rollup(baja:RelTime '3600000')`; //每小时一个rollup
    console.log(ordString);
    // @ts-ignore
    window.require &&
      // @ts-ignore
      window.requirejs(["baja!"], (baja: any) => {
        // 使用對應電表的 dataMap
        const dataMap = elecDataMaps.value[id][month];
        if (!dataMap) {
          console.warn(`未找到電表 ${id} (${month}) 的 dataMap`);
          return;
        }
        let lastTimestamp: string | null = null;
        let lastValue: number | null = null;
        baja.Ord.make(ordString).get({
          cursor: {
            before: () => {
              console.log(`開始訂閱 ${id} 的歷史資料`);
            },
            each: (record: any) => {
              const currentValue = record.get("min");
              const timestamp = record.get("timestamp").$cEncStr;
              if (
                currentValue !== null &&
                !isNaN(currentValue) &&
                currentValue !== 0 &&
                timestamp !== null
              ) {
                if (
                  lastTimestamp !== null &&
                  lastValue !== null &&
                  lastValue !== 0
                ) {
                  const diff = currentValue - lastValue;
                  dataMap.set(lastTimestamp, diff);
                }
                lastValue = currentValue;
                lastTimestamp = timestamp;
              }
            },
            after: () => {
              const elecCost = CalcuEleCost(dataMap);
              const eleceStandCost = CalcuEleStandCost(dataMap);
              console.log(`⏱️ 電表 ${id} 每小時差值 map：`, dataMap);
              if (item.slotPath) {
                // 确保 slotPath 存在
                elecCostSummary.value[month][id] = {
                  ...elecCost,
                  standCost: eleceStandCost.StandCost,
                  name: item.displayName,
                  area: item.area,
                };
                console.log(
                 `電表 ${item.displayName} 的電費計算結果 (${month})：`,
                  elecCostSummary.value[month][id]
                );
              } else {
                console.warn(
                  `電表 ${id} 的 slotPath 為空，無法儲存電費計算結果 (${month})`
                );
              }
            },
            limit: -1,
            offset: 0,
          },
        });
      });
  };


  // 定時更新資料的函數
  const updateHistoryData = () => {
    console.log("重新獲取歷史資料");
    if (elecData.value && elecData.value.length > 0) {
      elecCostSummary.value.thisMonth = {};
      elecCostSummary.value.lastMonth = {};

      elecData.value.forEach((item) => {
        const id = item.id;
        elecDataMaps.value[id] = {
          thisMonth: new Map<string, number>(),
          lastMonth: new Map<string, number>(),
        };
        getTimeToHistory(item, 'thisMonth');
        getTimeToHistory(item, 'lastMonth');
      });
    }
  };

  return {
    getElecDataFromBaja,
    elecData,
    elecCostSummary,
    startTime,
    endTime,
    updateHistoryData,
  };
});

export default useElecReportStore;
